@@ -1,6 +1,6 @@
 # -*- cperl; cperl-indent-level: 4 -*-
 # Copyright (C) 2009-2020, Roland van Ipenburg
-package HTML::Hyphenate v1.1.5;
+package HTML::Hyphenate v1.1.6;
 use Moose;
 use utf8;
 use 5.016000;
@@ -15,6 +15,7 @@ use Mojo::DOM;
 
 use Readonly;
 ## no critic qw(ProhibitCallsToUnexportedSubs)
+Readonly::Scalar my $EMPTY        => q{};
 Readonly::Scalar my $DOT          => q{.};
 Readonly::Scalar my $SOFT_HYPHEN  => qq{\N{SOFT HYPHEN}};
 Readonly::Scalar my $CLASS_JOINER => q{, .};                # for CSS classnames
@@ -65,7 +66,7 @@ after 'html' => sub {
             $self->_doctype( ${+}{doctype} );
         }
         else {
-            $self->_doctype();
+            $self->_doctype($EMPTY);
         }
     }
 };
@@ -143,9 +144,7 @@ sub _traverse_dom {
             }
         }
         elsif ( $TEXT eq $node->type || $RAW eq $node->type ) {
-            if ( !defined $self->_lang ) {
-                $self->_configure_lang($node);
-            }
+            $self->_configure_lang($node);
             $log->debug( sprintf $LOG{'TEXT_NODE'}, $node->to_string );
             $node->replace( $self->_hyphen( $node->to_string ) );
             return;
@@ -161,7 +160,7 @@ sub _clean_html {
     my ($self) = @_;
     my $html = $self->_dom->to_string();
     $self->_reset_dom;
-    if ( defined $self->_doctype ) {
+    if ( $EMPTY ne $self->_doctype ) {
         $html = $self->_doctype . $html;
     }
     return $html;
@@ -193,17 +192,31 @@ sub _hyphen_word {
     return $word;
 }
 
+sub _lang_attr {
+    my ($el) = @_;
+    if ( defined $el ) {
+        return $el->attr($LANG) || $el->attr(qq{xml:$LANG});
+    }
+    else {
+        return $el;
+    }
+}
+
 sub _configure_lang {
     my ( $self, $element ) = @_;
-    my $lang = $element->attr($LANG);
-    $lang ||= $element->attr(qq{xml:$LANG});
+    my $lang = _lang_attr($element);
+    if ( !defined $lang ) {
+        $lang = _lang_attr( $element->ancestors(qq{[$LANG]})->first() );
+    }
+    if ( !defined $lang ) {
+        $lang = $self->default_lang;
+    }
     my %hyphen_opts = (
         q{leftmin}  => $self->min_pre,
         q{rightmin} => $self->min_post,
     );
     defined $self->style
       && ( $hyphen_opts{'style'} = $self->style );
-    defined $lang || ( $lang = $self->default_lang );
     if ( !defined $self->_lang || $lang ne $self->_lang ) {
         $self->_lang($lang);
         $log->debug( sprintf $LOG{'LANGUAGE_SET'}, $lang );
@@ -289,11 +302,11 @@ __END__
 
 =head1 NAME
 
-HTML::Hyphenate - insert soft hyphens into HTML.
+HTML::Hyphenate - insert soft hyphens into HTML
 
 =head1 VERSION
 
-This document describes HTML::Hyphenate version v1.1.5.
+This document describes HTML::Hyphenate version v1.1.6.
 
 =head1 SYNOPSIS
 
@@ -384,7 +397,8 @@ Registers a TeX::Hyphen object to handle the language defined by C<lang>.
 
 =head1 CONFIGURATION AND ENVIRONMENT
 
-The output is generated by L<Mojo::DOM|Mojo::DOM>.
+The output is generated by L<Mojo::DOM|Mojo::DOM> so the environment variable
+C<MOJO_DOM_CSS_DEBUG> can be set to debug it's CSS selection process.
 
 =head1 DEPENDENCIES
 
@@ -432,8 +446,8 @@ TeX::Hyphen::Pattern
 hyphen somewhere in a word, and sometimes requires semantics to get it right.
 For example C<cafeetje> should be hyphenated as C<cafe-tje> and not
 C<cafee-tje> and C<buurtje> can be hyphenated as C<buur-tje> or C<buurt-je>,
-depending on it's meaning. While HTML could provide a bit more context –
-mainly the language being used – than plain text to handle these issues, the
+depending on it's meaning. While HTML could provide a bit more context -
+mainly the language being used - than plain text to handle these issues, the
 initial purpose of this module is to make it possible for HTML rendering
 engines that support soft hyphens to be able to break long words over multiple
 lines to avoid unwanted overflow.
